// CSEARCH.H

#if !defined ( _CSEARCH_H_ )

#define _CSEARCH_H_

#include <sys/timeb.h>		// struct timeb


#define SEARCH_MAX_MESSAGE_LENGTH			256

#define PROMOTION_WHITE							( DFLD1 | DFLD2 | DFLD3 | DFLD4 | DFLD5 )
#define PROMOTION_BLACK							( DFLD46 | DFLD47 | DFLD48 | DFLD49 | DFLD50 )

#define LEARN_DATABASE_FILE						"LearnDatabase.bin"

#define	HT_FIELD	6

#define SEARCH_MODE_NORMAL						0
#define SEARCH_MODE_ANALYSIS					1
#define SEARCH_MODE_MATCH						2

#define SEARCH_OUTPUT_CONSOLE					-1
#define SEARCH_OUTPUT_CONSOLE_INFO				-2
#define SEARCH_OUTPUT_CONSOLE_PVBOARD			-4

#define SEARCH_BLITZGAME						0
#define SEARCH_LONGGAME							1
#define SEARCH_FIXED_DEPTH						2
#define SEARCH_FIXED_TIME						3
#define SEARCH_MAXDEPTH							32

#define SEARCH_NODE_PV							0
#define SEARCH_NODE_ALL							-1
#define SEARCH_NODE_CUT							1

#define SEARCH_FHR_LEVEL0						80
#define SEARCH_FHR_LEVEL1						180
#define SEARCH_FHR_DEPTH0						2
#define SEARCH_FHR_DEPTH1						3

#define SEARCH_STARTSEARCHDEPTH					2
#define SEARCH_MCP_DEPTHREDUCTIONFACTOR			3
#define SEARCH_LMR_DEPTHREDUCTIONFACTOR			2
#define SEARCH_MCP_CHILD_SEARCH					5
#define SEARCH_MCP_CHILD_BETACUTOFF				2
#define SEARCH_MOVEGEN_MODE						2

// DamageTimeControl Definitions
#define SEARCH_TIMECONTROL_CONTINUE				0
#define SEARCH_TIMECONTROL_MULTIDRAWSCORE		1
#define SEARCH_TIMECONTROL_MULTIPOSITIVESCORE	2
#define SEARCH_TIMECONTROL_MULTICOUNT			5
#define SEARCH_TIMECONTROL_TIMEFRAC				40
#define SEARCH_TIMECONTROL_PANIC_DEPTH			10
#define SEARCH_TIMECONTROL_PANIC_DELTASCORE		20

#define SEARCH_PP_OUTPUT						0		// Default , no output
#define SEARCH_PP_NEXTMOVE_NONE					-1

#define PPSEARCH_OUTPUT_INIT					0
#define PPSEARCH_OUTPUT_START					1
#define PPSEARCH_OUTPUT_SEARCH					2
#define PPSEARCH_OUTPUT_RESEARCH				3
#define PPSEARCH_OUTPUT_EXIT					4

#define SP_OUTPUT_NONE							0
#define DEBUG_OUTPUT_NONE						0

#define PP_OUTPUT_NONE							0
#define PP_OUTPUT_SEARCHSTAT					1
#define PP_OUTPUT_SEARCHSP						2
#define PP_OUTPUT_SEARCHSPLIT					4
#define PP_OUTPUT_SEARCHABORT					8

#define MAXMOVE_BLITZGAME						80		// For Search Time Calculation
#define MINMOVE_BLITZGAME						60
#define TIMERESERVE_BLITZGAME					10

#define DEBUGSEARCHMESSAGE_MOVES				0
#define DEBUGSEARCHMESSAGE_SCORE				1
#define DEBUGSEARCHMESSAGE_SCORE_HASHREAD		2
#define DEBUGSEARCHMESSAGE_SCORE_EVALHASHREAD	3
#define DEBUGSEARCHMESSAGE_DEBUGDUMP			4

#define SEARCHMESSAGE_SEARCHINFO_TREE			0
#define SEARCHMESSAGE_SEARCHINFO_DEPTH			1
#define SEARCHMESSAGE_SEARCHINFO_NEXTMOVE		2
#define SEARCHMESSAGE_SEARCHINFO_SEARCH			3
#define SEARCHMESSAGE_SEARCHINFO_IASEARCH		4
#define SEARCHMESSAGE_SEARCHINFO_PV				5
#define SEARCHMESSAGE_SEARCHINFO_STRING			6
#define SEARCHMESSAGE_SEARCHINFO_SCOREROOT		7
#define SEARCHMESSAGE_SEARCHINFO_DATABASE		8
#define SEARCHMESSAGE_SEARCHINFO_DEPTHINFO		9
#define SEARCHMESSAGE_SEARCHINFO_DEBUG			10
#define SEARCHMESSAGE_SEARCHINFO_PP				11
#define SEARCHMESSAGE_SEARCHINFO_BOARD			12
#define SEARCHMESSAGE_SEARCHINFO_PVBOARD		13

// Learn Definitions
#define LEARN_HASHTABLE_READ_ERROR				0
#define LEARN_HASHTABLE_READ_MATCH				1
#define LEARN_NFLAGS							5
#define LEARN_WHITEFLAGS						0x1F
#define LEARN_BLACKFLAGS						0x3E0
#define LEARN_NSTAT								32
#define LEARN_WIN_VALUE							4500
#define LEARN_DRAW_VALUE						0
#define LEARN_LOOSE_VALUE						-4500
#define LEARN_NPOSITION							0
#define LEARN_WHITEWIN							1
#define LEARN_WHITEDRAW							2
#define LEARN_WHITELOOSE						3
#define LEARN_BLACKWIN							4
#define LEARN_BLACKDRAW							5
#define LEARN_BLACKLOOSE						6
#define LEARN_WIN								7
#define LEARN_DRAW								8
#define LEARN_LOOSE								9


class CBreakThrough64 ;
class CMoveGenerate64 ;

#ifdef _ENGINE_DAMAGE
class CEvaluation64 ;
#endif

class CHorizonEvaluation ; 
class CDatabase64 ;
class CBook64 ;


class CSearch64
{

public:

	CSearch64() ;
	~CSearch64() ;

	void Search64_Init(void) ;

	CString			m_csDatabasePath ;
	CString			m_csTablesPath ;

	HASHCODE		m_HashMove[4][4096] ;

	int		m_cix[64], m_cxi[51] ;

	int		m_iLearnStat[LEARN_NSTAT] ;
	int		m_iLearnIndex ;

	CBreakThrough64*	m_pBreakThrough64 ;
	CMoveGenerate64*	m_pMoveGenerate64 ;

#ifdef _ENGINE_DAMAGE
	CEvaluation64*		m_pEvaluation64 ;
#endif

	CHorizonEvaluation*	m_pHorizonEvaluation ;
	CDatabase64*		m_pDatabase64 ;
	CBook64*			m_pBook64 ;

	CRITICAL_SECTION	m_MPLock ;
	CMutex				m_SendEngineMessageLock ;

	float	fSearch64Time(PENGINEDATA) ;
	int		DamageTimeControl(PENGINEDATA) ;
	void	DamageTimeControl_Init(PENGINEDATA) ;
	void	OutputBoard(int, PENGINEDATA, PTSTACK) ;
	void	DisplayBoard(PENGINEDATA, CString*) ;

	int		SearchEvaluation(PENGINEDATA, bool, PTSTACK, int, int, bool) ;

	bool	bCombinationPattern(PTSTACK) ;

	void	Command(PENGINEDATA) ;
	void	DamageSearchTime(PSEARCHTHREADDATA, float*) ;
	void	LogOutput(PTSTACK, PSEARCHTHREADDATA, int, int, int) ;
	bool	PPQAvailableThread(PENGINEDATA, int, int) ;
	bool	PPQIdleThread(PENGINEDATA, int) ;
	bool	PPQStopThread(PENGINEDATA, int) ;
	void	PPWakeSleepingThreads(PENGINEDATA) ;

	int		PPSearchThreadIdle(PENGINEDATA) ;
	int		PPSearchThreadReady(PENGINEDATA) ;
	bool	PPSearchThreadStatus(PENGINEDATA, int) ;
	void	PPSearchThreadStart(PENGINEDATA, int) ;
	void	PPSearchThreadStat(PENGINEDATA) ;
	void	PPCopyTreeStack(int, int, int, PENGINEDATA ) ;

	void	BoardClear(PENGINEDATA) ;

	int		MoveGenerate(PENGINEDATA, bool) ;	// 64Bit MoveGenerator

	void	DisplayBoard(PENGINEDATA) ;

	void	DeConvertBoard64(PENGINEDATA) ;
	void	DeConvertBoard64(BITBOARD*, int*) ;
	void	ConvertBoard(bool, PENGINEDATA) ;
	void	ConvertBoard(PENGINEDATA) ;
	void	CopyBoard(PENGINEDATA) ;

	void	Decode_Move(int, int* ,PTSTACK, PLOOK) ;
	void	Execute_Move(PENGINEDATA) ;
	int		Find_Move(int*, PTSTACK, PLOOK) ;
	void	Format_Move64(PLOOK, int, CString* ) ;

	bool	Man_Move(PLOOK) ;

	void	Hash_Code64( PTSTACK ) ;
	void	Hash_ClearPP64( PENGINEDATA, bool ) ;

	HASHCODE	Search_MoveHash64(PTSTACK, PLOOK) ;
	void		Search_Move64(PTSTACK, PLOOK) ;

	void	PPSearchOutput(int, int, PENGINEDATA, PSPLITPOINT) ;
	void	PPSearchSP(int, PENGINEDATA, PSPLITPOINT) ;
	void	PPSearchSPIdleLoop(int, PENGINEDATA, PSPLITPOINT) ;

	int		Search(int, PENGINEDATA ) ;
	int		PPSearch(int, PENGINEDATA ) ;

	void	SearchPP_Clear(PENGINEDATA) ;
	void	Search_Close(void) ;

	void	Search_Init(PENGINEDATA) ;
	bool	Search_HashTable_Modify(PENGINEDATA) ;
	bool	Search_HashTable_Init(PENGINEDATA) ;
	bool	Search_Init_HTM(PENGINEDATA) ;
	bool	Search_Init_MoveHash64(void) ;

	void	MaterialValue(PTSTACK) ;

	int		Search_Time(PENGINEDATA) ;

	int		DatabaseSearch(int, int, int, PTSTACK, PENGINEDATA, bool) ;

private:

	int		Root_Find(int, PTSTACK, PROOT) ;
	void	Root_Init(PTSTACK, PROOT ) ;
	int		Root_Move64(PTSTACK, PSEARCHTHREADDATA, bool, bool, int, unsigned short int) ;
	void	Root_Sort(PTSTACK, PROOT ) ;

	bool	Draw64(PTSTACK ) ;

	void	PV_Calc(PENGINEDATA, PSEARCHTHREADDATA ) ;
	bool	SPPV_Copy(bool, PSPLITPOINT, PTSTACK) ;
	bool	PV_Copy(PTSTACK ) ;
	bool	PVPosition_Copy(int, int, PTSTACK ) ;

	void	History_Clear64(unsigned int* ) ;
	int		PPNextMove64(PSPLITPOINT, PTSTACK) ;
	void	History_Move64(PTSTACK, unsigned int* ) ;
	void	History_Update64(PTSTACK, unsigned int*, unsigned int ) ;

	bool	PPMulti_Cut_Pruning(int, int, int, int, int, PTSTACK, PENGINEDATA, bool) ;
	bool	Multi_Cut_Pruning(int, int, int, int, int, PTSTACK, PENGINEDATA, bool) ;

	int		PPIID(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;
	int		IID(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;

	PHARECPP Hash_ProbePP64(bool, HASHCODE, PSEARCHTHREADDATA pSearchThreadData, HASHDATA*) ;
	bool	bHash_Read_Score(bool, HASHDATA, unsigned int, int, int) ;
	int		Hash_ReadPP64_ETC(unsigned int, int, PTSTACK, PSEARCHTHREADDATA, int* ) ;

	void	Hash_Timestamp64(int, PENGINEDATA) ;
	void	Hash_TimestampPP64(int, PENGINEDATA) ;

	bool	Hash_WritePP64(int, unsigned int, int, int, PTSTACK, PSEARCHTHREADDATA, int ) ;

	void	Eval_Hash_Clear64(PEHAREC) ;
	bool	Eval_Hash_Read64(PTSTACK, PEHAREC, int* ) ;
	void	Eval_Hash_Write64(PTSTACK, PEHAREC, int ) ;

	bool	DatabaseMove(PTSTACK, PENGINEDATA) ;

	int		QSearch(int, int, PTSTACK, PSEARCHTHREADDATA, PENGINEDATA) ;
	int		QSearch_Next(int, int, int, PTSTACK, PSEARCHTHREADDATA, PENGINEDATA, bool, bool* ) ;

	bool	PPSearchSplit(int, PENGINEDATA, int, int, int*, int*, int*, int*) ;
	int		PPSearchN(int, int, int, int, PTSTACK, PENGINEDATA, bool) ;

	int		SearchN(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;

	int		PPSearchRoot(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;

	bool	IASearch1(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;
	int		Search1(int, int, int, int, PTSTACK, PENGINEDATA, bool ) ;

	bool	SearchDebug(PENGINEDATA, PTSTACK) ;
	void	SearchMessage(int, int, PENGINEDATA, int ) ;
	bool	DebugSearchMessage(int, int, PTSTACK, PENGINEDATA, int) ;
	void	SendEngineMessage(int, PENGINEDATA, int, CString) ;

} ;

#endif	// _CSEARCH_H_
